//  concatenate_test.cpp: test case for concatenate

//  Copyright Takeshi Mouri 2006.
//  Use, modification, and distribution are subject to the
//  Boost Software License, Version 1.0. (See accompanying file
//  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <hamigaki/iostreams/concatenate.hpp>
#include <boost/iostreams/device/array.hpp>
#include <boost/iostreams/read.hpp>
#include <boost/iterator/counting_iterator.hpp>
#include <boost/test/unit_test.hpp>
#include <algorithm>

namespace io_ex = hamigaki::iostreams;
namespace io = boost::iostreams;
namespace ut = boost::unit_test;

template<std::size_t N, class Source>
void check_array(char (&data)[N], Source src)
{
    typedef std::char_traits<char> traits_type;

    for (char i = 0; i < N; ++i)
        BOOST_CHECK_EQUAL(io::get(src), traits_type::to_int_type(i));

    for (char i = 0; i < N; ++i)
        BOOST_CHECK_EQUAL(io::get(src), traits_type::to_int_type(i));

    BOOST_CHECK_EQUAL(io::get(src), traits_type::eof());
}

void concatenate_test()
{
    char data[100];
    std::copy(
        boost::counting_iterator<char>(0),
        boost::counting_iterator<char>(100),
        &data[0]);

    check_array(
        data,
        io_ex::concatenate(io::array_source(data), io::array_source(data)));
}

ut::test_suite* init_unit_test_suite(int, char* [])
{
    ut::test_suite* test = BOOST_TEST_SUITE("concatenate test");
    test->add(BOOST_TEST_CASE(&concatenate_test));
    return test;
}
